import os

def get_if_qtip_is_dev():
    env = os.environ.get('QTIP_ENV')

    if (env == 'development'):
        return True

    return False

def get_server_path():
    if (get_if_qtip_is_dev()):
        return 'http://localhost:4225'

    return os.environ['QTIP_SERVER']


def get_auth_token_header():
    try:
        token = os.environ['QTIP_AUTH_TOKEN']
    except:
        raise NameError('QTIP_AUTH_TOKEN environment variable not set')

    return token



import sys
import os

# arg1 = action
# arg2 = path on website
# arg3 = file to upload
def get_action():
    return sys.argv[1]

def get_file_to_upload():
    arg_file_path = sys.argv[3]

    return os.path.join(os.getcwd(), arg_file_path)

def get_path_to_host():
    return sys.argv[2]



#!/usr/bin/env python3
import sys
import os
import json
import urllib.request
import urllib

from helpers import send_request
from helpers.args import get_action, get_file_to_upload, get_path_to_host
from helpers.env import get_if_qtip_is_dev, get_server_path, get_auth_token_header

from actions.preview.preview import init_preview
from actions.upload_and_serve_file import upload_and_serve_file

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'



def delete_file():
    path = get_path_to_host()
    request_path = get_server_path() + path

    request_headers = {'X-AUTH-TOKEN': get_auth_token_header()}
    request = urllib.request.Request(request_path, headers=request_headers)
    request.get_method = lambda: 'DELETE'
    send_request(request)

def invalid_action():
    print('Unknown action, type "qtip help" for more info')

def show_help():
    help_text = """
Here are the available actions for qtip:

    help - shows help text

    serve - uploads and serves a file
        Usage: qtip serve /file/path/on/server /path/to/file

    delete - removes a file from qtip server
        Usage: qtip remove /file/path/on/server
    """
    print(help_text)

# makes a file public to be viewed
def make_public():
    path = get_path_to_host()

    warning_text = f'{bcolors.FAIL}{bcolors.BOLD}You cannot make a project private again, you would need to delete the project to hide it.\n{bcolors.ENDC}'
    print(f'This will make the file `{path}` visible to everyone on the internet!\n\n{warning_text}')

    ans = input('Confirm making file public (y/n): ')

    if ans != 'y':
        print('Operation aborted')
        sys.exit()

    request_path = get_server_path() + path + '?method=make-public'
    request_headers = {'X-AUTH-TOKEN': get_auth_token_header()}
    request = urllib.request.Request(request_path, headers=request_headers)
    request.get_method = lambda: 'PUT'
    send_request(request)

def get_ip_list():
    ip_list_str = sys.argv[3]

    return ip_list_str.split(',')


def restrict_ip():
    path = get_path_to_host()
    restricted_ip_list = get_ip_list()

    request_path = get_server_path() + path + '?method=restrict-ip'

    request_headers = {'X-AUTH-TOKEN': get_auth_token_header(), 'content-type': 'application/json'}
    payload = {"ipList": restricted_ip_list}
    request_params = json.dumps(payload).encode('utf8')

    request = urllib.request.Request(request_path, headers=request_headers, data=request_params)
    request.get_method = lambda: 'PUT'
    send_request(request)

def list_directory():
    path = get_path_to_host()
    request_path = get_server_path() + path + '?method=ls'

    request_headers = {'X-AUTH-TOKEN': get_auth_token_header()}
    request = urllib.request.Request(request_path, headers=request_headers)
    request.get_method = lambda: 'GET'

    response = send_request(request)

    data = json.load(response)

    for result in data.get('results'):
        if 'public' in result:
            print(f'{bcolors.FAIL}{result}{bcolors.ENDC}')
        elif 'restricted' in result:
            print(f'{bcolors.WARNING}{result}{bcolors.ENDC}')
        else:
            print(result)

def get_file_data():
    path = get_path_to_host()
    request_path = get_server_path() + path + '?method=cat'

    request_headers = {'X-AUTH-TOKEN': get_auth_token_header()}
    request = urllib.request.Request(request_path, headers=request_headers)
    request.get_method = lambda: 'GET'

    response = send_request(request)

    data = json.load(response)

    return data.get('results')

def view_file():
    data = get_file_data()
    print(data)

def get_save_file_dest():
    try:
        arg_file_path = sys.argv[3]

        return os.path.join(os.getcwd(), arg_file_path)
    except:
        raise NameError('No path to save file exists')

def get_file():
    save_to_dest = get_save_file_dest()

    data = get_file_data()

    with open(save_to_dest, 'w') as file:
        file.write(data)


def map_actions_to_operations():
    action_map = {
        'serve': upload_and_serve_file,
        'delete': delete_file,
        'help': show_help,
        'public': make_public,
        'restrictip': restrict_ip,
        'ls': list_directory,
        'cat': view_file,
        'get': get_file,
        'preview': init_preview
    }

    action = get_action()

    func = action_map.get(action, lambda: invalid_action())

    func()

if __name__ == "__main__":
    if len(sys.argv) <= 1:
        print('No command specified, try `qtip help` for help!')
        sys.exit()

    map_actions_to_operations()


import os
import re
import json
import sys
import requests
import urllib.request
from helpers.env import get_if_qtip_is_dev, get_server_path, get_auth_token_header
from helpers.args import get_file_to_upload, get_path_to_host
from helpers import send_request

allowed_file_types = ['txt', 'md', 'json', 'js', 'tsx']



def upload_image(file_path):
    path = get_path_to_host()
    request_path = get_server_path() + '/___image'

    request_headers = {
        'X-AUTH-TOKEN': get_auth_token_header(),
    }

    url = request_path
    data = { 'path': path }

    files = {'file': open(file_path, 'rb')}
    r = requests.post(url, data=data, files=files, headers=request_headers)


def validate_file(file_name):
    obj = file_name.split('.')

    if len(obj) == 1:
        return

    if obj[-1] in allowed_file_types:
        return

    allowed_file_types_str = ', '.join(allowed_file_types)
    raise NameError(f'File is an invalid type. Allowed file extensions {allowed_file_types_str}')


def upload_and_serve_file(viaPublish):
    file_to_upload_path = get_file_to_upload()

    path = get_path_to_host()

    validate_file(file_to_upload_path)

    with open(file_to_upload_path, 'r') as content_file:
        content = content_file.read()

    image_details = []
    for imagePath in re.findall('(?:!\[.*?\]\(([^:]*?)\))', content):
        head, tail = os.path.split(imagePath)
        image_details.append({
            'originalPath': imagePath,
            'updatedAt': 0 #os.path.getmtime(imagePath)
        })


    payload = {"content": content, "imageDetails": image_details}

    request_params = json.dumps(payload).encode('utf8')
    request_headers = {'content-type': 'application/json', 'X-AUTH-TOKEN': get_auth_token_header()}
    request_path = get_server_path() + path
    public_url = request_path

    is_public = "-p" in sys.argv or viaPublish

    if is_public:
        request_path = request_path + '?forcePublic=true'

    request = urllib.request.Request(request_path, data=request_params,
                                     headers=request_headers)

    response = send_request(request)

    data = json.load(response)

    images_to_upload = data.get('imagesToUpload')

    if not images_to_upload or len(images_to_upload) == 0:
        print(f'This page is now served')

        if is_public:
            print(f'It should be available on when public :\n{public_url}')
        else:
            print(f'This page is not public yet! Please run the command:\n\nqtip public {get_path_to_host()} \n To make this page public to the world.')

        if not viaPublish:
            sys.exit()

    # upload images
    for imagePath in images_to_upload:
        upload_image(imagePath)


from flask import Flask, render_template, Response, request
import webbrowser
import logging
import os
import sys
import time
from actions.upload_and_serve_file import upload_and_serve_file

from helpers.args import get_file_to_upload, get_path_to_host
from helpers.env import get_server_path

public_folder = os.path.join(os.path.dirname(os.path.realpath(__file__)), '../../public')
cli_folder = os.path.join(os.path.dirname(os.path.realpath(__file__)))

app = Flask(__name__,
    template_folder=cli_folder,
    static_url_path='',
    static_folder=public_folder,
)

os.environ['WERKZEUG_RUN_MAIN'] = 'true'
log = logging.getLogger('werkzeug')
log.setLevel(logging.ERROR)


def watch_file():
    file = get_file_to_upload()
    # global file_content
    print(f'Watching file {file}')
    while True:
        with open(file, 'r') as file_data:
            # with data_lock:
            file_content = file_data.read()
            yield 'data: %s\n\n' % file_content.replace('\n', '|n')
            time.sleep(1)


@app.route('/publish', methods=['POST'])
def publish():
    response = Response()
    response.headers.add('Access-Control-Allow-Origin', '*')

    if request.method != 'POST':
        return response

    try:
        upload_and_serve_file(True)
    except:
        response.status_code = 400
        return response

    response.status_code = 200
    return response

@app.route('/content')
def stream():
    response = Response(watch_file(),
                        mimetype="text/event-stream",
                        )

    response.headers.add('Access-Control-Allow-Origin', '*')
    return response

@app.route('/')
def home():
    return render_template('preview.html', nav=get_path_to_host().split('/'))



def init_preview():
    url = 'http://localhost:5002'

    if "--live" in sys.argv:
        url = get_server_path() + get_path_to_host() + '?preview=1'

    print(f'You can edit your page on the following {url}')

    webbrowser.open(url)
    app.run(port=5002, threaded=True)



import urllib.request
import sys

def send_request(request):
    try:
        response = urllib.request.urlopen(request)

        return response
    except urllib.error.HTTPError as e:
        body = e.read().decode()
        print(f'Error: {body}')
        sys.exit()



from setuptools import setup, find_packages

print(find_packages('qtip-client-cli'))
setup(
    name='qtip-client-cli',
    packages=find_packages('qtip-client-cli'),
    py_modules=['qtip'],
    author='Shubham Naik',
    author_email='shub@shub.club',
    url='https://github.com/4shub/qtip',
    version='0.1.1-rev4',
    package_dir={'': 'qtip-client-cli' },
    install_requires=[
        'flask',
        'requests'
    ],
    entry_points= {
        "console_scripts": [
            "qtip = qtip:main",
        ]
    },
    setup_requires=[
        'setuptools>=41.0.1',
    ]
)